Config Enforce
==============

Config Enforce is intended to be used on production sites to ensure that configuration can't be altered directly. It does this by blocking changes to the active config and leveraging [Config Devel](https://drupal.org/project/config_devel) to automatically update active config when changes to exported YAML files are updated.

[Config Enforce Devel](https://drupal.org/project/config_enforce_devel) is the companion module which you install only in your development environment (see Security Model, below), and allows you to manage what config will be enforced (see Next Steps, below).

Installation
------------

Run the following commands:

```
composer require drupal/config_enforce
composer require --dev drupal/config_enforce_dev
composer config extra.enable-patching "true"
```

This should result in the following in your project's `composer.json`:

```
    "require": {
        "drupal/config_enforce": "^1.0",
[...]
    "require-dev": {
        "drupal/config_enforce_devel": "^1.0",
[...]
    "extra": {
        "enable-patching": true,
[...]
```

Then enable `config_enforce` by making it a dependency of your installation profile, a custom "site" module, on the modules admin page, or via Drush:

```
drush pm-enable -y config_enforce
```


Security Model
--------------

Config Enforce itself only applies exported configs under its control. It does not allow for the enforcement settings to be changed directly. This is an intentional element of its security model.

The Config Enforce Devel module provides a convenient UI to allow for managing enforced configs and their enforcement settings. It integrates with [Config Devel](https://drupal.org/project/config_devel) to automatically write configs to disk.

However, Config Enforce Devel is maintained as a seperate project, because it needs to be installed *only* in development environments. As such, you are *strongly discouraged* from deploying Config Enforce Devel outside of development environments.


Applying Enforced Configs
-------------------------

Once you've added some enforced configs (see Next Steps, below), you can import them by simply rebuilding caches:

```
drush cr
```

You can then check the log to see if any enforced configs have been imported.


Next steps
----------

To add, alter or remove enforced configs, see the [Config Enforce Devel README](https://git.drupalcode.org/project/config_enforce_devel/-/blob/1.0.x/README.md).


