<?php

namespace Drupal\config_enforce;

use Drupal\Core\Config\ConfigImporter as CoreConfigImporter;
use Drupal\Core\Config\InstallStorage;
use Drupal\Core\Config\StorageComparer;
use Drupal\config\StorageReplaceDataWrapper;

/**
 * Imports configuration.
 */
class ConfigImporter {

  // Use log() method and related traits.
  use ConfigEnforceHelperTrait;

  // Channel with which to log from this class.
  const LOGCHANNEL = 'config_enforce\ConfigImporter';

  /**
   * Basic constructor.
   */
  public function __construct() { }

  /**
   * Imports a config item from the given filename.
   *
   * @param string $filename
   *   The filename to import from with the Drupal-relative path.
   */
  public function importConfig($filenames) {
    $config_storage = \Drupal::service('config.storage');
    $source_storage = new StorageReplaceDataWrapper($config_storage);

    foreach ($this->getDataFromConfigFiles($filenames) as $config_name => $data) {
      $source_storage->replaceData($config_name, $data);
    }

    $storage_comparer = new StorageComparer($source_storage, $config_storage);
    $storage_comparer->createChangelist();

    // @TODO log whether config has changed?

    return $this->getConfigImporter($storage_comparer)->import();
  }

  protected function getDataFromConfigFiles($filenames) {
    $config_data = [];

    foreach ($filenames as $filename) {
      $contents = @file_get_contents($filename);
      $data = (new InstallStorage())->decode($contents);
      $config_name = basename($filename, '.yml');

      $this->handleSpecialCaseConfig($config_name, $data);

      $config_data[$config_name] = $data;
    }

    return $config_data;
  }

  /**
   * Return an instantiated ConfigImporter.
   */
  protected function getConfigImporter($storage_comparer) {
    // @TODO: simplify this when
    // https://www.drupal.org/project/drupal/issues/3123491 is fixed.
    return new CoreConfigImporter(
      $storage_comparer,
      \Drupal::service('event_dispatcher'),
      \Drupal::service('config.manager'),
      \Drupal::service('lock.persistent'),
      \Drupal::service('config.typed'),
      \Drupal::service('module_handler'),
      \Drupal::service('module_installer'),
      \Drupal::service('theme_handler'),
      \Drupal::service('string_translation'),
      \Drupal::service('extension.list.module')
    );
  }

  /**
   * Alter data or otherwise handle config special cases.
   */
  protected function handleSpecialCaseConfig($config_name, &$data) {
    switch ($config_name) {
      case 'system.site':
        // Site UUID gets validated, so we need to make sure it'll match.
        // @TODO log this?
        $data['uuid'] = \Drupal::config('system.site')->get('uuid');
        break;
      default:
        return;
    }
  }

}
